{% load cascade_tags %}{% spaceless %}

<uib-carousel interval="{{ instance.options.interval|default:5000 }}" {# style="{{ instance.inline_styles }}" #}>
	{% for slide in instance.child_plugin_instances %}
	<uib-slide>{% render_plugin slide %}</uib-slide>
	{% endfor %}
</uib-carousel>
{% endspaceless %}